@@ -10,10 +10,12 @@ from .commands.config import config_command
 from .commands.login import login_command
 from .commands.logout import logout_command
 from .commands.version import version_command
+from .utils.click_extensions import VCPCommandGroup, create_stub_command
+from .utils.dependencies import EXTRA_DESCRIPTIONS, check_extra_dependencies
 from .utils.version_check import check_for_updates_with_cache
 
 
-@click.group()
+@click.group(cls=VCPCommandGroup)
 @click.pass_context
 def cli(ctx):
     """VCP CLI - A command-line interface (CLI) to the Chan Zuckerberg Initiative's Virtual Cells Platform (VCP)"""
@@ -55,20 +57,40 @@ cli.add_command(cache_command, name="cache")
 cli.add_command(version_command, name="version")
 
 
-if config.feature_flags.model_command:
-    from .commands.model import model_command
+# Register optional commands based on feature flags and dependencies
+# Commands are either registered as fully functional (if deps installed)
+# or as stubs that show installation instructions (if deps missing)
 
-    cli.add_command(model_command)
+if config.feature_flags.model_command:
+    if check_extra_dependencies("model"):
+        from .commands.model import model_command
+
+        cli.add_command(model_command)
+    else:
+        # Register stub command that shows installation instructions
+        cli.add_command(
+            create_stub_command("model", "model", EXTRA_DESCRIPTIONS["model"])
+        )
 
 if config.feature_flags.data_command:
-    from .commands.data import data_command
+    if check_extra_dependencies("data"):
+        from .commands.data import data_command
 
-    cli.add_command(data_command)
+        cli.add_command(data_command)
+    else:
+        cli.add_command(create_stub_command("data", "data", EXTRA_DESCRIPTIONS["data"]))
 
 if config.feature_flags.benchmarks_command:
-    from .commands.benchmarks import benchmarks_command
+    if check_extra_dependencies("benchmarks"):
+        from .commands.benchmarks import benchmarks_command
 
-    cli.add_command(benchmarks_command, name="benchmarks")
+        cli.add_command(benchmarks_command, name="benchmarks")
+    else:
+        cli.add_command(
+            create_stub_command(
+                "benchmarks", "benchmarks", EXTRA_DESCRIPTIONS["benchmarks"]
+            )
+        )
 
 
 if __name__ == "__main__":
