@@ -8,11 +8,9 @@ import stat
 import subprocess
 import tempfile
 from pathlib import Path
-from typing import Dict, Optional, Tuple
+from typing import Any, Dict, Optional, Tuple
 from urllib.parse import urlparse
 
-from git import Repo
-
 from ...auth.github import GitHubAuth
 from ...config.config import Config
 
@@ -25,7 +23,7 @@ class GitOperations:
     def __init__(self, repo_path: str, config: Config, debug: bool = False):
         """Initialize with repository path and config for GitHub authentication."""
         self.repo_path = Path(repo_path)
-        self._repo: Optional[Repo] = None
+        self._repo: Optional[Any] = None  # Type will be git.Repo when available
         self.github_auth = GitHubAuth(config)
         self.debug = debug
         logger.debug(f"GitOperations initialized with repo_path={repo_path}")
@@ -87,11 +85,20 @@ class GitOperations:
         return None
 
     @property
-    def repo(self) -> Optional[Repo]:
+    def repo(self) -> Optional[Any]:
         """Get repository object with lazy loading."""
         if self._repo is None and (self.repo_path / ".git").exists():
             try:
+                from git import (  # noqa: PLC0415
+                    Repo,  # Lazy import for optional dependency
+                )
+
                 self._repo = Repo(self.repo_path)
+            except ImportError:
+                logger.error(
+                    "GitPython is not installed. Install with: pip install 'vcp-cli[model]'"
+                )
+                return None
             except Exception as e:
                 logger.error(f"Failed to initialize git repository: {e}")
                 return None
