@@ -5,6 +5,7 @@ import shutil
 import traceback
 from datetime import datetime
 from pathlib import Path
+from urllib.parse import urljoin
 
 import click
 import requests
@@ -405,7 +406,8 @@ def init_command(
                 work_dir = click.prompt("Work directory path")
 
     # Check if work directory exists, if not ask user if they want to create it
-    work_dir_path = Path(work_dir)
+    work_dir_path = Path(work_dir).expanduser()
+    work_dir = str(work_dir_path)
     if not work_dir_path.exists():
         console.print(f"[yellow]Work directory '{work_dir}' does not exist.[/yellow]")
 
@@ -533,7 +535,7 @@ def init_command(
     try:
         # Step 1: Check if model exists in model hub
         console.print("[yellow]Checking if model exists...[/yellow]")
-        check_url = f"{model_hub_url}/api/models/check"
+        check_url = urljoin(model_hub_url, "/api/models/check")
         check_params = {"model_name": model_name}
         _verbose_print(f"API check URL: {check_url}", verbose)
         _verbose_print(f"Check parameters: {check_params}", verbose)
@@ -688,7 +690,7 @@ def init_command(
             console.print("[yellow]Model doesn't exist. Creating new model...[/yellow]")
 
             # Call model hub API to create new submission
-            create_url = f"{model_hub_url}/api/sub"
+            create_url = urljoin(model_hub_url, "/api/sub")
             create_data = {
                 "model_name": model_name,
                 "model_version": model_version,
