@@ -6,6 +6,7 @@ import traceback
 from datetime import datetime
 from pathlib import Path
 from typing import Any, Dict, List, Optional
+from urllib.parse import urljoin
 
 import click
 import requests
@@ -25,6 +26,7 @@ from .utils import (
     check_merge_conflicts,
     get_git_status,
     validate_init_command_ran,
+    validate_metadata_files,
     validate_no_large_files,
     validate_stage_command_ran,
 )
@@ -35,6 +37,49 @@ console = Console()
 # Authentication functions removed - now using centralized GitHubAuth class
 
 
+def _metadata_passes_validation(
+    work_dir: str,
+    repo_root: Optional[Path],
+    config_data: Config,
+    verbose: bool,
+) -> bool:
+    """Validate metadata files and display results.
+
+    Args:
+        work_dir: Working directory path
+        repo_root: Git repository root path (if available)
+        config_data: Configuration object
+        verbose: Whether to show verbose output
+
+    Returns:
+        True if validation passed or was skipped, False if validation failed
+    """
+    # Use repo_root if available, otherwise use work_dir
+    validation_dir = str(repo_root) if repo_root else work_dir
+    if not validation_dir:
+        validation_dir = str(Path.cwd())
+
+    try:
+        result = validate_metadata_files(validation_dir, config_data, verbose)
+    except Exception as e:
+        console.print("[red]❌ Metadata validation error[/red]")
+        console.print(f"[red]{str(e)}[/red]")
+        console.print("\n[yellow]For detailed validation output, run:[/yellow]")
+        console.print("  [cyan]vcp model validate-metadata --verbose[/cyan]")
+        return False
+
+    if not result.success:
+        console.print("[red]❌ Metadata validation failed[/red]")
+        console.print(f"\n[red]{result.error_message}[/red]")
+        console.print("\n[yellow]To fix these errors:[/yellow]")
+        console.print("  • Run: [cyan]vcp model validate-metadata[/cyan]")
+        console.print("  • Fix the errors in your metadata files")
+        console.print("  • Try submit again")
+        return False
+
+    return True
+
+
 def push_model_branch(
     repo_path: str,
     branch_name: str,
@@ -162,6 +207,7 @@ This PR adds a new model to the VCP Model Hub.
 
 ### Validation
 - ✅ Init command completed
+- ✅ Metadata validation passed
 - ✅ Stage command completed
 - ✅ All files staged successfully
 - ✅ No large files detected
@@ -186,6 +232,7 @@ This PR updates the existing model with new changes.
 
 ### Validation
 - ✅ Init command completed
+- ✅ Metadata validation passed
 - ✅ Stage command completed
 - ✅ All files staged successfully
 - ✅ No large files detected
@@ -401,21 +448,35 @@ def validate_submission_data(data: Dict[str, Any]) -> Optional[str]:
     help="Path to the model repository work directory where model configuration is located",
 )
 @click.option("--skip-git", is_flag=True, help="Skip submission operations")
+@click.option(
+    "--skip-packaging",
+    is_flag=True,
+    help="Submit metadata only without packaging the model",
+)
+@click.option(
+    "--skip-metadata-validation",
+    is_flag=True,
+    hidden=True,
+    help="[DEBUG ONLY] Skip metadata validation",
+)
 def submit_command(
     config: Optional[str] = None,
     verbose: bool = False,
     work_dir: Optional[str] = None,
     skip_git: bool = False,
+    skip_packaging: bool = False,
+    skip_metadata_validation: bool = False,
 ):
     """Submit model for review with comprehensive validation.
 
     \b
     This command performs the following operations:
     1. Validates that init command was run (model configuration file exists)
-    2. Validates that stage command was run (only .ptr files in model_data)
-    3. Validates that no large files (>5GB) remain
-    4. Submits model data to VCP Model Hub API
-    5. Creates submission for model review
+    2. Validates metadata files format and required fields
+    3. Validates that stage command was run (only .ptr files in model_data)
+    4. Validates that no large files (>5GB) remain
+    5. Submits model data to VCP Model Hub API
+    6. Creates submission for model review
 
     \b
     The command reads submission data from:
@@ -478,9 +539,25 @@ def submit_command(
                 repo_root = parent
                 break
 
-        # Step 2: Validate that stage command was run
+        # Step 2: Validate metadata files
+        console.print("\n[bold blue]Step 2: Validating metadata files...[/bold blue]")
+        config_data = Config.load(config)
+
+        if skip_metadata_validation:
+            console.print("[dim]⚠️ Skipping metadata validation (debug mode)[/dim]")
+            metadata_valid = True
+        else:
+            metadata_valid = _metadata_passes_validation(
+                work_dir, repo_root, config_data, verbose
+            )
+        if not metadata_valid:
+            return
+
+        console.print("[green]✅ Metadata validation passed[/green]")
+
+        # Step 3: Validate that stage command was run
         console.print(
-            "\n[bold blue]Step 2: Validating stage command was run...[/bold blue]"
+            "\n[bold blue]Step 3: Validating stage command was run...[/bold blue]"
         )
 
         # Construct data path: output_path/*mlflow_pkg/model_data
@@ -501,9 +578,9 @@ def submit_command(
         console.print("[green]✅ Stage command validation passed[/green]")
         console.print(f"[blue]Model data path: {model_data_path}[/blue]")
 
-        # Step 3: Validate no large files
+        # Step 4: Validate no large files
         console.print(
-            "\n[bold blue]Step 3: Validating no large files remain...[/bold blue]"
+            "\n[bold blue]Step 4: Validating no large files remain...[/bold blue]"
         )
         if not validate_no_large_files(str(model_data_path)):
             console.print(
@@ -512,9 +589,6 @@ def submit_command(
         else:
             console.print("[green]✅ No large files validation passed[/green]")
 
-        # Load configuration early (needed for submission operations)
-        config_data = Config.load(config)
-
         # Get GitHub token early (needed for git operations)
         github_token = None
         if not skip_git:
@@ -540,7 +614,7 @@ def submit_command(
                 )
                 return
 
-        # Step 4: Create submission for review (if not skipped)
+        # Step 5: Create submission for review (if not skipped)
         if not skip_git:
             if not repo_root:
                 console.print("[red]Error: Could not find work directory root[/red]")
@@ -591,7 +665,7 @@ def submit_command(
                 commit_message = f"chore: Update model {model_name} {model_version}"
 
             # Get repository URL from model check API
-            model_check_url = f"{config_data.models.base_url}/api/models/check"
+            model_check_url = urljoin(config_data.models.base_url, "/api/models/check")
             model_check_params = {"model_name": model_name}
 
             # Get authentication headers for Model Hub API calls
@@ -887,14 +961,14 @@ def submit_command(
             console.print(json.dumps(safe_data, indent=2))
 
         # Submit to VCP Model Hub API
-        url = f"{config_data.models.base_url}/api/sub/submit"
+        url = urljoin(config_data.models.base_url, "/api/sub/submit")
 
         # Build request body - Always send model_name + model_version (API uses these to generate/validate ID)
         # Optionally include submission_id if available for validation
         request_body = {
             "model_name": metadata["model_name"],
             "model_version": metadata["model_version"],
-            "skip_packaging": False,  # Model needs packaging by default
+            "skip_packaging": skip_packaging,
         }
 
         submission_id = metadata.get("submission_id")
@@ -939,6 +1013,8 @@ def submit_command(
             # Update last_updated timestamp for resubmissions
             if metadata.get("resubmit", False):
                 try:
+                    # Track whether packaging was skipped in the most recent successful submission
+                    metadata["skip_packaging"] = skip_packaging
                     metadata["last_updated"] = datetime.utcnow().isoformat()
 
                     with open(metadata_path, "w") as f:
