@@ -8,10 +8,11 @@ import time
 from datetime import datetime
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple
-from urllib.parse import parse_qs, urlparse
+from urllib.parse import parse_qs, urljoin, urlparse
 
 import click
 import requests
+from pydantic import BaseModel
 from rich.console import Console
 from rich.progress import (
     BarColumn,
@@ -25,11 +26,48 @@ from rich.table import Table
 from ...auth.github import GitHubAuth
 from ...config.config import Config
 from ...utils.cache import get_vcp_cache_dir
+from ...utils.token import TokenManager
 from .models import ModelResponse
 
 console = Console()
 
 
+MODEL_METADATA_VALIDATION_ROUTE = "/api/metadata/validate"
+
+
+# Expected response structure from the model hub server
+class ValidationResult(BaseModel):
+    """Basic unit for validation results.
+
+    Pulled from vcp-model-hub, circa Sep 2025"""
+
+    valid: bool
+    human_readable_error: str | None = None
+
+
+class MetadataValidationResponse(BaseModel):
+    """Validation status and any error messages for each metadata provided.
+
+    Pulled from vcp-model-hub, circa Sep 2025"""
+
+    yaml_validation_result: ValidationResult
+    markdown_validation_result: ValidationResult
+
+
+class MetadataValidationResult(BaseModel):
+    """Result of metadata validation operation.
+
+    Attributes:
+        success: True if validation passed, False otherwise
+        error_message: Human-readable error message (None if success is True)
+        response_data: Optional response data from validation API (typically only present on validation failures)
+    """
+
+    success: bool
+    error_message: str | None = None
+    response_data: "MetadataValidationResponse | None" = None
+
+
 def validate_version_format(version: str) -> bool:
     """Validate version format against supported patterns."""
     # Pattern 1: v1 (simple version)
@@ -177,10 +215,8 @@ def download_multiple_files(
 
 # Production-grade file upload constants
 MAX_PARALLEL_UPLOADS = 5  # Number of concurrent uploads
-UPLOAD_TIMEOUT = 600  # 10 minutes base timeout per file (increased by 2x)
-MAX_UPLOAD_TIMEOUT = (
-    3600  # 60 minutes maximum timeout for very large files (increased by 2x)
-)
+UPLOAD_TIMEOUT = 3600  # 60 minutes base timeout per file
+MAX_UPLOAD_TIMEOUT = 21600  # 6 hours maximum timeout for very large files
 
 # Chunk size for downloads (16MB)
 CHUNK_SIZE = 16 * 1024 * 1024
@@ -418,7 +454,7 @@ def upload_single_file_to_presigned_url(
     max_retries: int = 3,
 ) -> bool:
     """
-    Upload a single file to S3 using a presigned URL.
+    Upload a single file to S3 using a presigned URL with streaming.
 
     Args:
         file_info: Dictionary containing file information including size_mb
@@ -432,75 +468,166 @@ def upload_single_file_to_presigned_url(
     """
     file_path = Path(file_info["absolute_path"])
     file_size_mb = file_info.get("size_mb", 0)
+    file_size_bytes = file_info.get("file_size", file_path.stat().st_size)
 
     # Calculate adaptive timeout based on file size
+    # For very large files, allow more time
     base_timeout = UPLOAD_TIMEOUT
-    adaptive_timeout = min(base_timeout + (file_size_mb * 2), MAX_UPLOAD_TIMEOUT)
 
-    for attempt in range(max_retries + 1):
+    # Enhanced timeout calculation for large files
+    if file_size_bytes > 5 * 1024 * 1024 * 1024:  # 5GB
+        # For large files, use more aggressive timeout calculation
+        # Base timeout + 1 minute per 100MB + minimum 2 hours
+        adaptive_timeout = min(
+            base_timeout + (file_size_mb * 0.6),  # 0.6 minutes per MB
+            MAX_UPLOAD_TIMEOUT,  # Cap at 6 hours
+        )
+    else:
+        adaptive_timeout = min(base_timeout + (file_size_mb * 2), MAX_UPLOAD_TIMEOUT)
+
+    console.print(
+        f"[blue]Staging {file_path.name} ({file_size_mb:.2f} MB) with timeout {adaptive_timeout}s...[/blue]"
+    )
+
+    # Enhanced retry logic for large files
+    max_attempts = max_retries + 1
+    if file_size_bytes > 5 * 1024 * 1024 * 1024:  # 5GB
+        # For large files, allow more retry attempts
+        max_attempts = max(max_attempts, 5)  # Minimum 5 attempts for large files
+        console.print(
+            f"[blue]Using enhanced retry logic: {max_attempts} attempts for large file[/blue]"
+        )
+
+    for attempt in range(max_attempts):
+        # Create a fresh session for each attempt to avoid connection reuse issues
+        # This prevents a timed-out or failed connection from affecting subsequent uploads
+        session = requests.Session()
+
+        # Configure session for better SSL handling
+        session.verify = True  # Ensure SSL verification is enabled
+        session.mount(
+            "https://",
+            requests.adapters.HTTPAdapter(
+                max_retries=0,  # Disable requests-level retries, we handle them manually
+                pool_connections=0,  # Disable connection pooling to prevent SSL reuse
+                pool_maxsize=0,  # No connections in pool
+            ),
+        )
+
+        # Force fresh connection for retry attempts
+        if attempt > 0:
+            # Close any existing connections to prevent SSL reuse
+            session.close()
+            session = requests.Session()
+            session.verify = True
+            session.mount(
+                "https://",
+                requests.adapters.HTTPAdapter(
+                    max_retries=0,
+                    pool_connections=0,  # Disable connection pooling
+                    pool_maxsize=0,  # No connections in pool
+                ),
+            )
+
         try:
             if verbose and attempt > 0:
                 console.print(
-                    f"[yellow]Upload attempt {attempt + 1}/{max_retries + 1} for {file_path.name}[/yellow]"
+                    f"[yellow]Upload attempt {attempt + 1}/{max_attempts} for {file_path.name}[/yellow]"
                 )
 
-            # Read file content
-            with open(file_path, "rb") as f:
-                file_content = f.read()
-
-            console.print(
-                f"[blue]Read {len(file_content)} bytes from {file_path.name}[/blue]"
-            )
-            console.print(f"[blue]Staging with timeout {adaptive_timeout}s...[/blue]")
-
             # Set proper headers for S3 uploads
             headers = {
                 "Content-Type": "application/octet-stream",
-                "Content-Length": str(len(file_content)),
+                "Content-Length": str(file_size_bytes),
             }
 
-            response = requests.put(
-                presigned_url,
-                data=file_content,
-                headers=headers,
-                timeout=adaptive_timeout,
-            )
+            # Use streaming upload - don't load entire file into memory
+            # This keeps memory usage low and maintains connection
+            with open(file_path, "rb") as f:
+                response = session.put(
+                    presigned_url,
+                    data=f,  # Stream from file object
+                    headers=headers,
+                    timeout=(
+                        30,
+                        adaptive_timeout,
+                    ),  # 30s connect, adaptive_timeout read
+                )
 
             console.print(
                 f"[blue]Upload response for {file_path.name}: {response.status_code}[/blue]"
             )
 
             if response.status_code == 200:
+                console.print(f"[green]Successfully uploaded {file_path.name}[/green]")
                 return True
             else:
                 console.print(
-                    f"Access denied ({response.status_code}) but URL not expired: {response.text}"
+                    f"[yellow]Upload failed with status {response.status_code}: {response.text}[/yellow]"
                 )
                 if response.status_code in [400, 403, 404]:
                     # Permanent failures - don't retry
                     console.print(
-                        f"Upload failed for {file_path.name} with status {response.status_code}: {response.text}"
+                        f"[red]Permanent failure for {file_path.name} with status {response.status_code}[/red]"
                     )
                     return False
 
         except requests.exceptions.Timeout:
             console.print(
-                f"Upload timeout for {file_path.name} on attempt {attempt + 1}"
+                f"[yellow]Upload timeout for {file_path.name} on attempt {attempt + 1}/{max_retries + 1}[/yellow]"
             )
             if attempt < max_retries:
-                time.sleep(2**attempt)  # Exponential backoff
+                # Exponential backoff with cap
+                backoff_time = min(2**attempt, 60)  # Cap at 60 seconds
+                console.print(f"[blue]Waiting {backoff_time}s before retry...[/blue]")
+                time.sleep(backoff_time)
                 continue
             else:
+                console.print(
+                    f"[red]Max retries exceeded for {file_path.name} due to timeout[/red]"
+                )
+                break
+
+        except requests.exceptions.SSLError:
+            console.print(
+                f"[yellow]SSL connection error for {file_path.name} on attempt {attempt + 1}/{max_attempts}[/yellow]"
+            )
+            if attempt < max_attempts - 1:
+                # Longer backoff for SSL errors with exponential increase
+                backoff_time = min(
+                    2 ** (attempt + 2), 300
+                )  # Cap at 5 minutes for SSL errors
+                console.print(
+                    f"[blue]Waiting {backoff_time}s before retry (SSL connection issue)...[/blue]"
+                )
+                time.sleep(backoff_time)
+                continue
+            else:
+                console.print(
+                    f"[red]Max retries exceeded for {file_path.name} due to SSL connection issues[/red]"
+                )
                 break
 
         except Exception as e:
-            console.print(f"Exception uploading {file_path.name}: {str(e)}")
-            if attempt < max_retries:
-                time.sleep(2**attempt)
+            console.print(
+                f"[yellow]Exception uploading {file_path.name} on attempt {attempt + 1}/{max_attempts}: {str(e)}[/yellow]"
+            )
+            if attempt < max_attempts - 1:
+                backoff_time = min(2**attempt, 60)
+                console.print(f"[blue]Waiting {backoff_time}s before retry...[/blue]")
+                time.sleep(backoff_time)
                 continue
             else:
+                console.print(
+                    f"[red]Max retries exceeded for {file_path.name}: {str(e)}[/red]"
+                )
                 break
 
+        finally:
+            # Always close the session to free up the connection
+            # This ensures each upload attempt gets a completely fresh connection
+            session.close()
+
     return False
 
 
@@ -570,11 +697,7 @@ def upload_files_batch_parallel(
             file_info_with_size = file_info.copy()
             file_info_with_size["size_mb"] = file_size_mb
 
-            console.print(
-                f"[blue]Staging {file_info['relative_path']} ({file_size_mb:.2f} MB)...[/blue]"
-            )
-
-            # Upload the file
+            # Upload the file (it will print its own status message)
             success = upload_single_file_to_presigned_url(
                 file_info_with_size,
                 presigned_url,
@@ -762,7 +885,7 @@ def validate_init_command_ran(
                 console.print("To initialize a model workspace:")
                 console.print("  [blue]vcp model init[/blue]")
                 console.print()
-                console.print("Or specify all parameters:")
+                console.print("Or specify parameters:")
                 console.print(
                     "  [blue]vcp model init --model-name <name> --model-version <version> --license-type <license> --work-dir <path>[/blue]"
                 )
@@ -794,7 +917,7 @@ def validate_init_command_ran(
                 console.print("To initialize a model workspace:")
                 console.print("  [blue]vcp model init[/blue]")
                 console.print()
-                console.print("Or specify all parameters:")
+                console.print("Or specify parameters:")
                 console.print(
                     "  [blue]vcp model init --model-name <name> --model-version <version> --license-type <license> --work-dir <path>[/blue]"
                 )
@@ -981,6 +1104,165 @@ def validate_no_large_files(data_path: str) -> bool:
     return True
 
 
+def _truncate_text_for_display(text: str, max_chars: int = 5000) -> str:
+    """Truncate text for display if it exceeds max_chars.
+
+    Args:
+        text: Text to potentially truncate
+        max_chars: Maximum characters before truncation
+
+    Returns:
+        Original text or truncated text with indicator
+    """
+    if len(text) <= max_chars:
+        return text
+
+    truncated = text[:max_chars]
+    chars_remaining = len(text) - max_chars
+    return f"{truncated}[dim]... ({chars_remaining} more characters)[/dim]"
+
+
+def _parse_validation_error_response(
+    response: requests.Response,
+) -> MetadataValidationResult:
+    """Parse a 422 validation error response from the API.
+    This will be the validation errors that the user needs to fix.
+
+    Args:
+        response: Response object with status_code 422
+
+    Returns:
+        MetadataValidationResult with error details
+
+    Raises:
+        Exception: If response JSON cannot be parsed into expected schema
+    """
+    # Parse and validate response structure
+    try:
+        resp_data = MetadataValidationResponse(**response.json())
+    except Exception as e:
+        raise ValueError(
+            f"Validation failed but could not parse error response: {e}"
+        ) from e
+
+    # Build error messages from validation results
+    errors = []
+    if not resp_data.yaml_validation_result.valid:
+        yaml_error = (
+            resp_data.yaml_validation_result.human_readable_error or "Unknown error"
+        )
+        errors.append(
+            f"YAML Validation Failed:\n{yaml_error}\n\n"
+            f"[blue]Tip: To skip validation for optional fields, comment them out completely (e.g., # field_name: value)[/blue]"
+        )
+
+    if not resp_data.markdown_validation_result.valid:
+        markdown_error = (
+            resp_data.markdown_validation_result.human_readable_error or "Unknown error"
+        )
+        errors.append(f"Markdown Validation Failed:\n{markdown_error}")
+
+    error_message = "\n\n".join(errors) if errors else "Validation failed"
+    return MetadataValidationResult(
+        success=False,
+        error_message=error_message,
+        response_data=resp_data,
+    )
+
+
+def _parse_validation_response(response: requests.Response) -> MetadataValidationResult:
+    """Parse validation API response into a MetadataValidationResult.
+
+    Args:
+        response: Response object from validation API
+
+    Returns:
+        MetadataValidationResult based on response status
+    """
+    try:
+        err_data = response.json()
+        error_detail = err_data.get("detail", response.text)
+    except Exception:
+        error_detail = response.text
+
+    return MetadataValidationResult(
+        success=False,
+        error_message=f"Validation request failed (status {response.status_code}): {error_detail}",
+    )
+
+
+def validate_metadata_files(
+    work_dir: str, config_data: Config, verbose: bool = False
+) -> MetadataValidationResult:
+    """
+    Validate metadata files by calling the VCP Model Hub validation endpoint.
+
+    This function is reusable by both the validate-metadata command and the submit command.
+
+    Args:
+        work_dir: Path to the model repository work directory
+        config_data: Configuration object with API base URL
+        verbose: Whether to print verbose debug information
+
+    Returns:
+        MetadataValidationResult with validation status and any error details
+
+    Raises:
+        FileNotFoundError: If metadata files don't exist
+        requests.RequestException: If API request fails
+    """
+    # Determine work directory and file paths
+    work_path = Path(work_dir).expanduser()
+    yaml_path = work_path / "model_card_docs" / "model_card_metadata.yaml"
+    markdown_path = work_path / "model_card_docs" / "model_card_details.md"
+
+    # Get authentication headers
+    token_manager = TokenManager()
+    headers = token_manager.get_auth_headers(include_content_type=False)
+    if not headers:
+        raise RuntimeError("Not logged in. Please run 'vcp login' first.")
+
+    # Prepare validation request
+    endpoint = urljoin(config_data.models.base_url, MODEL_METADATA_VALIDATION_ROUTE)
+
+    # Field names must match FastAPI server expectations
+    _YAML_FIELD_NAME = "yaml_file"
+    _MARKDOWN_FIELD_NAME = "markdown_file"
+
+    if verbose:
+        console.print("\n[bold blue]Request Details:[/bold blue]")
+        console.print(f"Endpoint: {endpoint}")
+        console.print(f"YAML file: {yaml_path}")
+        console.print(f"Markdown file: {markdown_path}")
+
+    # Make validation request
+    with open(yaml_path, "rb") as yf, open(markdown_path, "rb") as mf:
+        files = {
+            _YAML_FIELD_NAME: (yaml_path.name, yf, "application/x-yaml"),
+            _MARKDOWN_FIELD_NAME: (markdown_path.name, mf, "text/markdown"),
+        }
+        response = requests.post(endpoint, files=files, headers=headers)
+
+    if verbose:
+        console.print(
+            f"\n[bold blue]Response status:[/bold blue] {response.status_code}"
+        )
+        console.print("[bold blue]Raw response text:[/bold blue]")
+        console.print(_truncate_text_for_display(response.text), highlight=False)
+
+    # If successful, return success result
+    if response.status_code == 200:
+        return MetadataValidationResult(success=True)
+
+    # If validation errors are returned
+    elif response.status_code == 422:
+        return _parse_validation_error_response(response)
+
+    # All other errors (400, 500, etc.)
+    else:
+        return _parse_validation_response(response)
+
+
 def check_large_file_considerations(files_info: List[Dict]) -> Dict:
     """
     Check for large files (>5GB) and provide S3 presigned URL considerations.
@@ -1761,3 +2043,43 @@ def format_models_table(models: list[ModelResponse]) -> Table:
             table.add_row(model.name, "No versions", "N/A")
 
     return table
+
+
+def get_model_metadata_path(work_dir: str | Path | None) -> Path:
+    """Return the path to the `.model-metadata` JSON file. Path is not guaranteed to exist."""
+    work_dir = Path(work_dir).expanduser() if work_dir else Path.cwd()
+    return Path(work_dir) / ".model-metadata"
+
+
+def load_model_metadata(work_dir: str | Path | None) -> dict[str, Any]:
+    """
+    Load the `.model-metadata` JSON file as a dict. Returns an empty dict if the file does not exist.
+    FIXME: DRY up the rest of the codebase using this function (currently the same logic is duplicated in many places).
+    FIXME: This should return a pydantic model instead of a dict.
+    """
+    metadata_path = get_model_metadata_path(work_dir)
+    if not metadata_path.exists():
+        return {}
+    with open(metadata_path, "r") as f:
+        return json.load(f)
+
+
+def update_model_metadata(
+    work_dir: str | Path | None, **kwargs: dict[str, str | bool]
+) -> None:
+    """
+    Update the `.model-metadata` JSON file with the given key-value pairs. Creates the file if it does not exist.
+    FIXME: DRY up the rest of the codebase using this function (currently the same logic is duplicated in many places).
+    FIXME: The updated metadata should be validated before it is written to disk (once that validation exists).
+    """
+    metadata_path = get_model_metadata_path(work_dir)
+    # Create the file if it doesn't exist
+    if not metadata_path.exists():
+        with open(metadata_path, "w") as f:
+            json.dump(kwargs, f)
+    # Else update the existing json
+    else:
+        metadata = load_model_metadata(work_dir)
+        metadata.update(kwargs)
+        with open(metadata_path, "w") as f:
+            json.dump(metadata, f, indent=2)
