@@ -238,7 +238,7 @@ def search_data_api(
     cursor: Optional[str] = None,
     exact: bool = False,
 ) -> SearchResponse:
-    processed_term = _process_search_term(term, exact)
+    processed_term = _process_search_term(term, exact) + " AND latest_version:true"
 
     params = {
         "query": processed_term,
